"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createCli = void 0;
const arg_1 = __importDefault(require("arg"));
const path_1 = __importDefault(require("path"));
const chalk_1 = __importDefault(require("chalk"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const runMigration_1 = require("../runMigration");
/**
 *
 * @param options.binName the name of the bin file, as presented in the --help output
 * @param options.migrationFile absolute path to the migrationFile
 * @param options.version path to the verison of the CLI (package.json's version)
 * @param options.subCommands create sub commands for multiple migrations on the same application
 */
const createCli = async ({ binName, migrationFile: customMigrationFile, version, subCommands, }) => {
    process.on('unhandledRejection', (error) => {
        throw error;
    });
    const args = arg_1.default({
        // Types
        '--version': Boolean,
        '--help': Boolean,
        '--verbose': Boolean,
        '--cwd': String,
        '--dry': Boolean,
        '--yes': Boolean,
        '--reportFile': String,
        '--reporter': String,
        // Aliases
        '-v': '--version',
        '-h': '--help',
        '-d': '--dry',
        '-y': '--yes',
    }, {
        permissive: false,
    });
    if (args['--version']) {
        console.log(version);
        process.exit(0);
    }
    const subCommand = args._[0];
    let migrationFile;
    if (subCommand && subCommands) {
        if (subCommand in subCommands) {
            migrationFile = subCommands[subCommand].migrationFile;
        }
        else {
            console.log(chalk_1.default.red `unknown command ${chalk_1.default.bold(subCommand)}`);
            console.log(chalk_1.default.red `Please use one of the following sub commands:`);
            Object.keys(subCommands).forEach((subCommand) => {
                console.log(chalk_1.default.red ` > ${subCommand}`);
            });
            process.exit(1);
        }
    }
    else {
        migrationFile = customMigrationFile || args._[0];
    }
    let helpUsage = binName;
    if (subCommands) {
        const subCommandsString = Object.keys(subCommands).join('|');
        if (migrationFile) {
            helpUsage += ` [${subCommandsString}]`;
        }
        else {
            helpUsage += ` <${subCommandsString}>`;
        }
    }
    else if (!migrationFile) {
        helpUsage += ' <path/to/migration.ts>';
    }
    if (args['--help']) {
        console.log(`
        Usage
          $ ${helpUsage}

        Options
          --version, -v   Version number
          --help, -h      Displays this message
          --dry, -d       Dry-run mode, does not modify files
          --yes, -y       Skip all confirmation prompts. Useful in CI to automatically answer the confirmation prompt
          --cwd           Runs the migration on this directory [defaults to process.cwd()]
          --reporter      Choose a reporter ("default"/"quiet"/"markdown"/"path/to/custom/reporter")
          --reportFile    Create a markdown report and output it to a file [for example "report.md"]
      `);
        process.exit(0);
    }
    let migrationFileAbsolutePath;
    if (!path_1.default.isAbsolute(migrationFile)) {
        migrationFileAbsolutePath = path_1.default.join(process.cwd(), migrationFile);
    }
    else {
        migrationFileAbsolutePath = migrationFile;
    }
    if (!fs_extra_1.default.existsSync(migrationFileAbsolutePath)) {
        console.error(chalk_1.default.red(`couldn't find a migration file on "${migrationFileAbsolutePath}"`));
        process.exit(1);
    }
    const customCwd = args['--cwd'];
    const cwdAbsolutePath = customCwd && path_1.default.join(process.cwd(), customCwd);
    if (cwdAbsolutePath && !fs_extra_1.default.existsSync(cwdAbsolutePath)) {
        console.error(chalk_1.default.red(`passed cwd "${cwdAbsolutePath}" does not exists`));
        process.exit(1);
    }
    const cwd = cwdAbsolutePath || process.cwd();
    await runMigration_1.runMigration({
        cwd,
        migrationFilePath: migrationFileAbsolutePath,
        dry: !!args['--dry'],
        yes: !!args['--yes'],
        reportFile: args['--reportFile'],
        reporter: args['--reporter'],
    });
};
exports.createCli = createCli;
